//
//  main.m
//  TPPreciseTimerTest
//
//  Created by Michael Tyson on 06/09/2011.
//  Copyright 2011 A Tasty Pixel. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "TPPreciseTimer.h"
#import <mach/mach_time.h>

@interface TestObject : NSObject
- (void)timerFired:(NSTimer*)timer;
@end
@implementation TestObject
- (void)timerFired:(NSTimer*)timer {
    void (^block)(void) = [timer userInfo];
    block();
}
- (void)shouldNotFire {
    printf("This should never happen. Badness 10000/n");
}
@end

int main (int argc, const char * argv[])
{

    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    
    struct mach_timebase_info timebase;
    mach_timebase_info(&timebase);
    double timebase_ratio = ((double)timebase.numer / (double)timebase.denom) * 1.0e-9;
    
    TestObject *testObject = [[[TestObject alloc] init] autorelease];

    for ( int i=0; i<10; i++ ) {        
        NSTimeInterval duration = ((double)rand()/RAND_MAX) * 2.0;
        printf("\nDuration: %lf s\n", duration);

        NSTimeInterval start = mach_absolute_time() * timebase_ratio;

        [TPPreciseTimer scheduleBlock:^{
            NSTimeInterval end = mach_absolute_time() * timebase_ratio;
            printf("TPPreciseTimer deviation:\t%lf s\n", (end-start) - duration);
        } inTimeInterval:duration];
        
        [NSTimer scheduledTimerWithTimeInterval:duration 
                                         target:testObject 
                                       selector:@selector(timerFired:) 
                                       userInfo:^{
                                           NSTimeInterval end = mach_absolute_time() * timebase_ratio;
                                           printf("NSTimer deviation:\t\t\t%lf s\n", (end-start) - duration);
                                       }repeats:NO];
        
        [[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:duration + 0.1]];
    }
    
    NSTimeInterval start = mach_absolute_time() * timebase_ratio;
    [TPPreciseTimer scheduleBlock:^{
        NSTimeInterval end = mach_absolute_time() * timebase_ratio;
        printf("TPPreciseTimer test pre-empt: Event 2 deviation:\t%lf s\n", (end-start) - 0.4);
    } inTimeInterval:0.4];
    
    [NSThread sleepForTimeInterval:0.1];
    
    start = mach_absolute_time() * timebase_ratio;
    [TPPreciseTimer scheduleBlock:^{
        NSTimeInterval end = mach_absolute_time() * timebase_ratio;
        printf("TPPreciseTimer test pre-empt: Event 1 deviation:\t%lf s\n", (end-start) - 0.2);
    } inTimeInterval:0.2];

    [TPPreciseTimer scheduleAction:@selector(shouldNotFire) target:testObject inTimeInterval:0.2];
    [NSThread sleepForTimeInterval:0.1];
    [TPPreciseTimer cancelAction:@selector(shouldNotFire) target:testObject];
    
    [NSThread sleepForTimeInterval:0.5];
    
    [pool drain];
    return 0;
}

